AssassinAbiTree = Object:subclass('AssassinAbiTree')
local AbiButton = Button:subclass('AbiButton')
files = love.filesystem.enumerate('assets/characters')
character = {}
for i,v in ipairs(files) do
	if love.filesystem.isFile('assets/characters/'..v) then
		local f = v:gmatch("(%w+).(%w+)")
		local file,ext=f()
		if ext=='png' then
			character[file] = love.graphics.newImage('assets/characters/'..v)
		end
	end
end

function AbiButton:initialize(group,x,y,w,h,skill)
	super.initialize(self,group,x,y,w,h)
	self.skill = skill
	self.face = character[self.skill.name]
	--print (64*self.face:getHeight()/self.face:getWidth())
	--self.quad = love.graphics.newQuad(0,0,64,64*self.face:getHeight()/self.face:getWidth(),self.face:getWidth(),self.face:getHeight())
	self.upgradetime = 0
end

function AbiButton:draw()
	if self.hide then return end
	if self.skill.level>0 then
		if self.particle then love.graphics.draw(self.particle,0,0) end
		if self.upgradetime > 0 then love.graphics.draw(systems[5],0,0) end
	end
	local scale = 64/self.face:getHeight()
	love.graphics.draw(self.face,self.x,self.y,0,scale,scale)
	love.graphics.setColor(0,0,0,255)
	love.graphics.printf('LEVEL'..self.skill.level,self.x+self.w/2,self.y+self.h,0,'center')
	love.graphics.setColor(255,255,255,255)
	if self.hover then
	love.graphics.setColor(0,0,0,255)
	love.graphics.printf(self.skill:getdescription(),50,love.graphics.getHeight()-150,love.graphics.getWidth()-100,'left')
	love.graphics.printf(self.skill:getdescriptiondata(),50,love.graphics.getHeight()-150,love.graphics.getWidth()-100,'right')
	love.graphics.setColor(255,255,255,255)
	end
end

function AbiButton:click()
	if self.group.enabletime>0 then return end
	self.upgradetime = 3
	enables = self.skill:setLevel(self.skill:getLevel()+1)
	if enables and #enables>0 then
		for k,v in pairs(enables) do
			self.group:enableabi(self,self.group.buttongroup[v])
		end
	end
end

function AbiButton:update(dt)
	if self.hide then return end
	super.update(self,dt)
	if self.skill.level>0 and self.particle then
		self.particle:setPosition(self.x+self.w/2,self.y+self.h/2)
		self.particle:start()
		self.particle:update(dt)
	end
	if self.upgradetime>0 then
		self.upgradetime = self.upgradetime - dt
		if self.upgradetime > 2 then
			systems[5]:setPosition(self.x+self.w/2,self.y+self.h/2)
			systems[5]:start()
		end	
		systems[5]:update(dt)
	end
	if self.hover then
		
	end
end

local background = love.graphics.newImage('assets/qmsht.jpg')
local scroll = love.graphics.newImage('assets/scroll.png')
local scrollbackground = love.graphics.newImage('assets/scrollbackground.jpg')
function AssassinAbiTree:initialize(unit)
	self.unit = unit
	self.buttongroup = {}
	self.buttongroup[unit.skills.pistol] = AbiButton:new(self,75,75,64,64,unit.skills.pistol)
	self.buttongroup[unit.skills.stunbullet] = AbiButton:new(self,160,75,64,64,unit.skills.stunbullet)
	self.buttongroup[unit.skills.explosivebullet] = AbiButton:new(self,245,75,64,64,unit.skills.explosivebullet)
	self.buttongroup[unit.skills.momentumbullet] = AbiButton:new(self,330,75,64,64,unit.skills.momentumbullet)
	--self.buttongroup[unit.skills.pistol].particle = systems[1]
	for k,v in pairs(self.buttongroup) do
		if v.skill.level<=0 then v.hide = true end
	end
	self.enabletime = 0
end
function AssassinAbiTree:show()
	self.time = 100
	self.dt = 0
	self.folddt = 100
end
function AssassinAbiTree:fold()
	self.folddt = 0
end

function AssassinAbiTree:enableabi(originbutton,targetbutton)
	self.enabletime = 4
	self.enableco = {originbutton.x+originbutton.w/2,originbutton.y+originbutton.h/2}
	self.stepco = {(targetbutton.x-originbutton.x),(targetbutton.y-originbutton.y)}
	self.targetbutton = targetbutton
end
function AssassinAbiTree:update(dt)
	self.dt = self.dt+dt
	if self.folddt<1.1 then self.folddt = self.folddt+dt end
	for k,v in pairs(self.buttongroup) do
		v:update(dt)
	end	
	if self.enabletime > 0 then
		if self.enabletime > 2 then
			systems[6]:setPosition(unpack(self.enableco))
			systems[6]:start()
			self.enableco = {self.enableco[1]+self.stepco[1]*dt/2,self.enableco[2]+self.stepco[2]*dt/2}
		else
			self.targetbutton.hide=nil
		end
		systems[6]:update(dt)
		self.enabletime = self.enabletime - dt
	end
end
function AssassinAbiTree:draw()
	if self.dt < 1.1 then
		map:draw()
		map.camera:revert()
		love.graphics.setScissor(0,0,love.graphics.getWidth()*self.dt,love.graphics.getHeight())
		love.graphics.draw(background,-self.dt/self.time*background:getWidth(),0,0,1,1)
		
		love.graphics.draw(scrollbackground,0,love.graphics.getHeight()-180,0,1,1)
		for k,v in pairs(self.buttongroup) do
			v:draw()
		end
		love.graphics.draw(scroll,love.graphics.getWidth()*self.dt-50,0,0,1,1)
		love.graphics.setScissor()
	elseif self.folddt < 1 then
		map:draw()
		map.camera:revert()
		
		love.graphics.setScissor(0,0,love.graphics.getWidth()*(1-self.folddt),love.graphics.getHeight())
		love.graphics.draw(background,-self.dt/self.time*background:getWidth(),0,0,1,1)
		
		love.graphics.draw(scrollbackground,0,love.graphics.getHeight()-180,0,1,1)
		for k,v in pairs(self.buttongroup) do
			v:draw()
		end
		love.graphics.draw(scroll,love.graphics.getWidth()*(1-self.folddt)-50,0,0,1,1)
		love.graphics.setScissor()
		if self.folddt>=0.9 then
			popsystem()
		end
	else
		love.graphics.draw(background,-self.dt/self.time*background:getWidth(),0,0,1,1)
		love.graphics.draw(scrollbackground,0,love.graphics.getHeight()-180,0,1,1)
		for k,v in pairs(self.buttongroup) do
			v:draw()
		end
		if self.enabletime > 0 then love.graphics.draw(systems[6]) end
	end
end