Map=Object:subclass('Map')

--[[
a list of type of unit and corresponding data
					CategoryID	Mask	Group
item				1			34567
doodad				2			56
player				3			1357
enemy				4			1467
player missle		5			12357
enemy missles		6			12467
dead				7			13456
]]--

typeinfo = {
	item = {1,{3,4,5,6,7}},
	doodad = {2,{5,6}},
	player = {3,{3,5,7}},
	enemy = {4,{4,6,7}},
	playermissle = {5,{5}},
	enemymissle = {6,{5}},
	dead = {7,{5,6}}
}

safezone = {1500,1500}

function Map:initialize(w,h)
	self.world = love.physics.newWorld(-w/2,-h/2,w/2,h/2)
	self.world:setCallbacks(nil,nil,persist)
	self.units = {}
--	self.missles = {}
--	self.deads = {}
	self.doodads = {}
end
--[[
function Map:addMissle(...)
	for k,unit in ipairs(arg) do
		self.missles[unit] = true
		if unit.createBody then unit:createBody(self.world) end
	end
end

function Map:removeMissle(...)
	for k,unit in ipairs(arg) do
		self.missles[unit] = nil
	end
end

function Map:addDead(...)
	for k,unit in ipairs(arg) do
		self.deads[unit] = true
		if unit.createBody then unit:createBody(self.world) end
	end
end

function Map:removeDead(...)
	for k,unit in ipairs(arg) do
		self.deads[unit] = nil
	end
end
]]--

function handlepersist()
	for k,v in ipairs(collides) do
		local a,b,c=unpack(v)
		if map.units[a] and map.units[b] then
			if a.persist then
				a:persist(b,c)
			end
			if b.persist then
				b:persist(a,c)
			end
		end
	end
end

function persist(a,b,coll)
	table.insert(collides,{a,b,coll})
end

function Map:addUnit(...)
	for k,unit in ipairs(arg) do
		self.units[unit] = true
		if unit.createBody then unit:createBody(self.world) end
	end
end

function Map:removeUnit(...)
	for k,unit in ipairs(arg) do
		self.units[unit] = nil
	end
end

function Map:update(dt)
	collides = {}
	self.world:update(dt)
	handlepersist()
	for unit,v in pairs(self.units) do
		if unit.update then unit:update(dt) end
	end
end

function Map:draw()
	if self.camera then self.camera:apply() end
	if self.background then self.background:draw() end
	for unit,v in pairs(self.units) do
		if unit.draw then unit:draw() end
	end
end

function Map:findUnitsInArea(area)
	if area.type == 'circle' then
		return self:findUnitsWithCondition(
			function(unit) 
				return withincirclearea(unit,area.x,area.y,area.range)
			end)
	end
end

function Map:findUnitsWithCondition(func)
	result = {}
	for unit,v in pairs(self.units) do
		if func(unit) then
			table.insert(result,unit)
		end
	end
	return result
end

function normalize(x,y)
	local n = math.sqrt(x*x+y*y)
	if n==0 then return 0,0 end
	return x/n,y/n
end

function withinrectanglearea(unit,x,y,w,h)
	return unit.x>=x and unit.x<=x+w and unit.y>=y and unit.y<=y+h
end

function withincirclearea(unit,x,y,r)
	return getdistance(unit,{x=x,y=y})<r
end

function withinfanarea(unit,x,y,r,angle,range)
	local angle2 = math.atan2(unit.y-y,unit.x-x)
	if angle*angle2<0 then
		angle = math.abs(angle+angle2)
	else
		angle = math.abs(angle2-angle)
	end
	return getdistance(unit,{x=x,y=y})<r and angle<range
end

function getdistance(a,b)
	x,y=a.x-b.x,a.y-b.y
	return math.sqrt(x*x+y*y)
end
