Unit = Object:subclass('Unit')
bufftable = {}
bufftable.__mode = 'k'
function Unit:initialize(x,y,rad,mass)
	self.x,self.y=x,y
	self.rad = rad
	self.mass = mass
	self.state = 'slide'
	self.order = 'stop'
	self.direction = {1,0}
	self.movingforce = self.mass*50
	self.speedlimit = 20000
	self.buffs={}
	setmetatable(self.buffs,bufftable)
end

function Unit:damage()
end

function Unit:getHP()
	if self.hp then return self.hp end
	return 1000
end

function Unit:getHPPercent()
	if self.hp and self.maxhp then return self.hp/self.maxhp end
	return 1
end

function Unit:createBody(world)
	self.body = love.physics.newBody(world,self.x,self.y,self.mass,self.mass)
	self.shape = love.physics.newCircleShape(self.body,0,0,self.rad)
	if self.controller then
		category,masks = unpack(typeinfo[self.controller])
		self.shape:setCategory(category)
		self.shape:setMask(unpack(masks))
	end
	self.shape:setData(self)
end

function Unit:loadDefaultAllowance()
	self.allowmovement = true
	self.allowskill = true
	self.allowmelee = true
	self.movementspeedbuff = 0
	self.movementspeedbuffpercent = 1
--	self.spellspeedbuff = 0
	self.spellspeedbuffpercent = 1
	self.timescale = 1
end

function Unit:applyBehavior(k,b)
	self[k]=b
end

function Unit:update(dt)
	self:loadDefaultAllowance()
	for k,v in pairs(self.buffs) do
		if k.buff then
			k:buff(self,dt)
		end
	end
	-- all the buff/debuffs
	local speedlimit = (self.speedlimit + self.movementspeedbuff) * self.movementspeedbuffpercent
	if self.body then
		if self.skill and self.allowskill then
			if self.skill.cd then self.skill:cd(dt) end
			self.skill:update(dt*self.spellspeedbuffpercent)
		end
		if self.state == 'stop' then
			self.body:setLinearVelocity(0,0)
			self.body:setAngularVelocity(0)
		elseif self.state == 'slide' then
			local x,y = self.body:getLinearVelocity()
			if x*x+y*y < 100 then
				self.body:setLinearVelocity(0,0)
				self.body:setAngularVelocity(0)
			else
				if x > 0 then x = x - 500*dt else x = x + 500*dt end
				if y > 0 then y = y - 500*dt else y = y + 500*dt end
				self.body:setLinearVelocity(x,y)
			end
		elseif self.state == 'move' and self.allowmovement then
			local x,y = self.body:getLinearVelocity()
			if x*x+y*y < speedlimit then
				x,y = unpack(self.direction)
				x,y = x*self.movingforce,y*self.movingforce
				self.body:applyForce(x,y)
			else
				self.body:setLinearVelocity(x*0.9,y*0.9)
			end
		end
		self.x,self.y=self.body:getPosition()
	end
end

function Unit:draw()
	love.graphics.circle('fill',self.x,self.y,self.rad,32)
end
